//==============================================================================
//
// Copyright (c) 2007 Ryan Winter. All rights reserved.
//
// This file may be used under the terms of the GNU General Public
// License version 2.0 appearing in the file LICENSE packaged with this file.
//
//==============================================================================

#include "fractanoid.h"

#include "ui_about.h"

#include <widget/fractal_widget.h>
#include <config/configure_dialog.h>
#include <config/window_config.h>

#include <QtGui>

#include <iostream>

//------------------------------------------------------------------------------
// public:
//------------------------------------------------------------------------------
Fractanoid::Fractanoid()
{
    ui.setupUi(this);

    m_fractal_widget = new FractalWidget();
    setCentralWidget(m_fractal_widget);

    createActions();

    // restore the window geometry
    WindowConfig wc;
    restoreGeometry(wc.geometry);
}

//------------------------------------------------------------------------------
// private slots:
//------------------------------------------------------------------------------
void
Fractanoid::aboutFractanoid()
{
    QDialog *dialog = new QDialog(this);

    Ui::About about;
    about.setupUi(dialog);

    dialog->exec();
}

//------------------------------------------------------------------------------
void
Fractanoid::configure()
{
    ConfigureDialog* dialog = new ConfigureDialog(this);
    dialog->exec();
}

//------------------------------------------------------------------------------
// protected:
//------------------------------------------------------------------------------
void
Fractanoid::closeEvent(QCloseEvent* event)
{
    // save the window geometry
    WindowConfig wc;
    wc.geometry = saveGeometry();
    wc.save();

    QWidget::closeEvent(event);
}

//------------------------------------------------------------------------------
// private:
//------------------------------------------------------------------------------
void
Fractanoid::createActions()
{
    // configure the actions
    ui.actionPanMode->setChecked(true);

    // connect the actions
    connect(ui.actionQuit,     SIGNAL(triggered()), this, SLOT(close()));
    connect(ui.actionConfig,   SIGNAL(triggered()), this, SLOT(configure()));
    connect(ui.actionAbout,    SIGNAL(triggered()), this, SLOT(aboutFractanoid()));

    connect(ui.actionZoomIn,   SIGNAL(triggered()), m_fractal_widget,  SLOT(zoomIn()));
    connect(ui.actionZoomOut,  SIGNAL(triggered()), m_fractal_widget,  SLOT(zoomOut()));
    connect(ui.actionPanMode,  SIGNAL(triggered()), m_fractal_widget,  SLOT(panMode()));
    connect(ui.actionZoomMode, SIGNAL(triggered()), m_fractal_widget,  SLOT(zoomMode()));

    connect(ui.actionPanMode,  SIGNAL(triggered()), ui.actionZoomMode, SLOT(toggle()));
    connect(ui.actionZoomMode, SIGNAL(triggered()), ui.actionPanMode,  SLOT(toggle()));

    // setup the toolbar
    QToolBar *toolbar = new QToolBar(this);
    toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

    toolbar->addAction(ui.actionPanMode);
    toolbar->addAction(ui.actionZoomMode);
    toolbar->insertSeparator(ui.actionZoomIn);
    toolbar->addAction(ui.actionZoomIn);
    toolbar->addAction(ui.actionZoomOut);

    addToolBar(toolbar);

    // setup the menu
    QMenu *menu_file      = menuBar()->addMenu(tr("&File"));
    QMenu *menu_edit      = menuBar()->addMenu(tr("&Edit"));
//    QMenu *menu_bookmarks = menuBar()->addMenu(tr("&Bookmarks"));
    QMenu *menu_help      = menuBar()->addMenu(tr("&Help"));

    menu_file->addAction(ui.actionQuit);
    menu_edit->addAction(ui.actionConfig);
    menu_help->addAction(ui.actionAbout);
}
