//==============================================================================
//
// Copyright (c) 2007 Ryan Winter. All rights reserved.
//
// This file may be used under the terms of the GNU General Public
// License version 2.0 appearing in the file LICENSE packaged with this file.
//
//==============================================================================

#include "gradient.h"

#include <algorithm>
#include <cassert>

// define sorting of the Points
bool operator<(const Gradient::Point &lhs, const Gradient::Point &rhs)
{
    return lhs.index < rhs.index;
}

//------------------------------------------------------------------------------
// public:
//------------------------------------------------------------------------------
Gradient::Gradient(const std::vector<Point> &points)
    :
    m_points(points)
{
    assert(m_points.size() >= 2);

    // sort the vector of points by index
    std::sort(m_points.begin(), m_points.end());
}

//------------------------------------------------------------------------------
Gradient::~Gradient()
{
}

//------------------------------------------------------------------------------
std::vector<QRgb>
Gradient::swab(unsigned int no_points) const
{
    const float min_index = m_points[0].index;
    const float max_index = m_points[m_points.size() - 1].index;
    const float delta     = (max_index - min_index) / no_points;

    std::vector<QRgb> gradient;

    unsigned int index = 0;

    for (unsigned int ii = 0; ii < no_points; ++ii)
    {
        float point = min_index + ii * delta;

        // move to the next section of the gradient
        while (point > m_points[index + 1].index)
        {
            ++index;
        }

        if (index >= m_points.size())
        {
            // handle the case of falling off the end a bit
            gradient.push_back(m_points[m_points.size() - 1].colour.rgb());
        }
        else
        {
            const float  point1  = m_points[index].index;
            const float  point2  = m_points[index + 1].index;
            const QColor colour1 = m_points[index].colour;
            const QColor colour2 = m_points[index + 1].colour;

            const float percent = (point - point1) / (point2 - point1);

            float r = colour1.red()   + percent * (colour2.red()   - colour1.red());
            float g = colour1.green() + percent * (colour2.green() - colour1.green());
            float b = colour1.blue()  + percent * (colour2.blue()  - colour1.blue());

            QColor colour(static_cast<int>(r), static_cast<int>(g), static_cast<int>(b));
            gradient.push_back(colour.rgb());
        }
    }

    return gradient;
}
