//==============================================================================
//
// Copyright (c) 2007 Ryan Winter. All rights reserved.
//
// This file may be used under the terms of the GNU General Public
// License version 2.0 appearing in the file LICENSE packaged with this file.
//
//==============================================================================

#ifndef fractanoid_widget_fractal_thread_pool_h
#define fractanoid_widget_fractal_thread_pool_h

#include "fractal_widget_thread.h"

#include <QObject>
#include <QMutex>
#include <QSemaphore>

#include <vector>

class FractalThreadPool : public QObject
{
  Q_OBJECT

  public:
    FractalThreadPool();
    virtual ~FractalThreadPool();

    void setCpuNumber(int cpus);
    void queue(const CellDescriptor& cell);

  signals:
    void processingComplete(CellImage* image);

  private:
    //--------------------------------------------------------------------------
    // Description : Not allowed for this class
    //--------------------------------------------------------------------------
    FractalThreadPool &operator=(const FractalThreadPool &rhs);
    FractalThreadPool(const FractalThreadPool &rhs);

    void clearThreads();

    typedef std::vector<FractalWidgetThread*> WidgetThreads;

    QMutex             m_lock;
    QSemaphore         m_data_access;
    WidgetThreads      m_threads;
    CellDescriptorList m_cells;
};

#endif

