//==============================================================================
//
// Copyright (c) 2007 Ryan Winter. All rights reserved.
//
// This file may be used under the terms of the GNU General Public
// License version 2.0 appearing in the file LICENSE packaged with this file.
//
//==============================================================================

#ifndef fractanoid_widget_fractal_widget_h
#define fractanoid_widget_fractal_widget_h

#include <widget/fractal_thread_pool.h>

#include <QGraphicsView>

class RubberBand;

class FractalWidget : public QGraphicsView
{
  Q_OBJECT

  public:
    FractalWidget(QWidget *parent = 0);

    void configure();

  protected:
    virtual void scrollContentsBy(int dx, int dy);
    virtual void showEvent(QShowEvent* event);

    // mouse interaction
    virtual void wheelEvent(QWheelEvent* event);
    virtual void mouseMoveEvent(QMouseEvent* event);
    virtual void mousePressEvent(QMouseEvent* event);
    virtual void mouseReleaseEvent(QMouseEvent* event);

  private slots:
    void panMode();
    void zoomMode();
    void zoomIn();
    void zoomOut();

    void addCell(CellImage*);
    void buildScene();

  private:
    enum Mode
    {
        Mode_Pan,
        Mode_Zoom,
    };

    void zoom(float zoom);
    void zoom(QRect zoom_geometry);
    void clear();

    float             m_zoom;
    float             m_rotation;
    int               m_version;

    QGraphicsScene    m_scene;

    Mode              m_mode;

    RubberBand*       m_rubber_band;

    QRect             m_bounding;
    QPointF           m_centre;

    FractalThreadPool m_thread_pool;
};

#endif
