//==============================================================================
//
// Copyright (c) 2007 Ryan Winter. All rights reserved.
//
// This file may be used under the terms of the GNU General Public
// License version 2.0 appearing in the file LICENSE packaged with this file.
//
//==============================================================================

#include "rubber_band.h"

#include <QMouseEvent>

#include <cmath>

//------------------------------------------------------------------------------
// public:
//------------------------------------------------------------------------------
RubberBand::RubberBand(QWidget* parent)
    :
    QRubberBand(QRubberBand::Rectangle, parent)
{
}

//------------------------------------------------------------------------------
void
RubberBand::setOrigin(QPoint origin)
{
    // cache the origin
    m_origin = origin;

    setGeometry(QRect(m_origin, QSize()));
    show();
}

//------------------------------------------------------------------------------
void
RubberBand::update(QRect rect, QPoint pos)
{
    if (isVisible())
    {
        float ratio      = 1.0 * rect.width() / rect.height();
        float sqrt_ratio = std::sqrt(ratio);

        QPoint delta = pos - m_origin;

        float x   = std::abs(delta.x());
        float y   = std::abs(delta.y());
        float mag = 2.0 * x * y / (x + y);

        int width  = static_cast<int>(mag * sqrt_ratio);
        int height = static_cast<int>(mag / sqrt_ratio);

        // fix the quadrant
        if (delta.x() < 0) width  *= -1;
        if (delta.y() < 0) height *= -1;

        QRect selection(m_origin, QSize(width, height));

        setGeometry(selection.normalized());
    }
}

//------------------------------------------------------------------------------
void
RubberBand::setDestination()
{
    hide();
}
